/* Mini link */
.minilink {
	display: inline-block;
	padding: .125em .375em;
	text-transform: uppercase;
	font-size: 0.875rem; /* 14px /16 */
	text-decoration: none;
	background-color: #ddd;
	border-radius: 0.1875em; /* 3px /16 */
	font-weight: 500;
	margin: 0 0.4285714285714em 0.07142857142857em 0; /* 0 6px 1px 0 /14 */
	line-height: 1.285714285714; /* 18px /14 */
	font-family: system-ui, -apple-system, sans-serif;
}
@media (prefers-color-scheme: dark) {
  .minilink {
    background-color: #222;
    /*
      !important to override .elv-callout a
      see _includes/components/callout.css
     */
    color: #fff !important;
  }
}
table .minilink {
	margin-top: 6px;
}
.minilink[href] {
	box-shadow: 0 1px 1px 0 rgba(0,0,0,.5);
}
.minilink[href]:hover,
.minilink[href]:focus {
	background-color: #bbb;
}
@media (prefers-color-scheme: dark) {
  .minilink[href]:hover,
  .minilink[href]:focus {
    background-color: #444;
  }
}
pre + .minilink {
	color: #fff;
	border-radius: 0 0 0.2857142857143em 0.2857142857143em; /* 4px /14 */
	float: right;
	background-color: #444;
	color: #fff;
}
pre[class*=language-] + .minilink {
	position: relative;
	top: -0.7142857142857em; /* -10px /14 */
}
p.minilink {
	float: right;
	margin-left: 2em;
	margin-bottom: 2em;
}
h1 .minilink,
h2 .minilink,
h3 .minilink,
h4 .minilink {
	font-size: 0.9375rem; /* 15px /16 */
	vertical-align: middle;
	margin-left: 1em;
}
h3 .minilink,
h4 .minilink {
	font-size: 0.8125rem; /* 13px /16 */
}
.minilink + pre[class*=language-] {
	clear: both;
}

.minilink-addedin {
	text-transform: none;
	box-shadow: 0 0 0 1px rgba(0,0,0,0.3);
}
@media (prefers-color-scheme: dark) {
  .minilink-addedin {
    box-shadow: 0 0 0 1px rgba(255,255,255,0.3);
  }
}
.minilink-addedin:not(:first-child) {
	margin-left: .5em;
}
.minilink-addedin.minilink-inline {
	margin: 0 4px;
	background-color: #fff;
}

.minilink-lower {
	text-transform: none;
	background-color: transparent;
}
.minilink-lower[href] {
	box-shadow: 0 0 0 1px rgba(0,0,0,0.5);
}
.minilink-lower[href]:hover,
.minilink-lower[href]:focus {
	background-color: #eee;
}

.minilink > .minilink {
	margin: -.125em .375em -.125em -.375em;
	box-shadow: none;
	border-top-right-radius: 0;
	border-bottom-right-radius: 0;
}
