/* Warning */
.elv-info {
	line-height: 1.5;
	padding: 0.8125em 1em 0.75em; /* 13px 16px 12px /16 */
	margin-left: -1rem;
	margin-right: -1rem;
	margin-bottom: 2em;
	background-color: #dff7ff;
}
.elv-info:before {
	content: "ℹ️ ";
}
.elv-info-warn {
	background-color: #ffa;
}
.elv-info-warn:before {
	content: "⚠️ ";
}
.elv-info:first-child {
	margin-top: 0;
}
body > .elv-info {
	margin-left: 0;
	margin-right: 0;
	padding: .5rem 1rem;
}
@media (min-width: 37.5em) and (min-height: 25em) { /* 600px / 400px */
	body > .elv-info-sticky {
		position: sticky;
		top: 0;
		z-index: 2;
		box-shadow: 0 3px 0 0 rgba(0,0,0,.08);
	}
}